@@ -1,29 +1,26 @@
 # This file is part of Tryton.  The COPYRIGHT file at the top level of
 # this repository contains the full copyright notices and license terms.
 from dateutil.relativedelta import relativedelta
-from trytond.model import ModelView, ModelSQL, fields
-from trytond.wizard import Wizard, StateView, StateTransition, StateAction, \
-    Button
+from trytond.model import ModelView, ModelSQL, Workflow, fields
+from trytond.wizard import Wizard, StateView, StateAction, Button
 from trytond.tools import datetime_strftime
 from trytond.pyson import Eval, If, PYSONEncoder
 from trytond.transaction import Transaction
 from trytond.pool import Pool
 
-__all__ = ['FiscalYear',
-    'BalanceNonDeferralStart', 'BalanceNonDeferral',
-    'CloseFiscalYearStart', 'CloseFiscalYear']
+__all__ = ['FiscalYear', 'FiscalYearLine',
+    'BalanceNonDeferralStart', 'BalanceNonDeferral']
 
 STATES = {
-    'readonly': Eval('state') == 'close',
+    'readonly': Eval('state') != 'open',
 }
 DEPENDS = ['state']
 
 
-class FiscalYear(ModelSQL, ModelView):
+class FiscalYear(Workflow, ModelSQL, ModelView):
     'Fiscal Year'
     __name__ = 'account.fiscalyear'
     name = fields.Char('Name', size=None, required=True, depends=DEPENDS)
-    code = fields.Char('Code', size=None)
     start_date = fields.Date('Starting Date', required=True, states=STATES,
         domain=[('start_date', '<=', Eval('end_date', None))],
         depends=DEPENDS + ['end_date'])
@@ -33,9 +30,10 @@ class FiscalYear(ModelSQL, ModelView):
     periods = fields.One2Many('account.period', 'fiscalyear', 'Periods',
             states=STATES, depends=DEPENDS)
     state = fields.Selection([
-        ('open', 'Open'),
-        ('close', 'Close'),
-        ], 'State', readonly=True, required=True)
+            ('open', 'Open'),
+            ('close', 'Close'),
+            ('locked', 'Locked'),
+            ], 'State', readonly=True, required=True)
     post_move_sequence = fields.Many2One('ir.sequence', 'Post Move Sequence',
             required=True, domain=[('code', '=', 'account.move'),
                 ['OR',
@@ -52,6 +50,9 @@ class FiscalYear(ModelSQL, ModelView):
             ('id', If(Eval('context', {}).contains('company'), '=', '!='),
                 Eval('context', {}).get('company', -1)),
             ], select=True)
+    close_lines = fields.Many2Many('account.fiscalyear-account.move.line',
+            'fiscalyear', 'line', 'Close Lines')
+    icon = fields.Function(fields.Char("Icon"), 'get_icon')
 
     @classmethod
     def __setup__(cls):
@@ -72,6 +73,11 @@ class FiscalYear(ModelSQL, ModelView):
                 'reopen_error': ('You can not reopen fiscal year "%s" until '
                     'you reopen all later fiscal years.'),
                 })
+        cls._transitions |= set((
+                ('open', 'close'),
+                ('close', 'locked'),
+                ('close', 'open'),
+                ))
         cls._buttons.update({
                 'create_period': {
                     'invisible': ((Eval('state') != 'open')
@@ -87,6 +93,9 @@ class FiscalYear(ModelSQL, ModelView):
                 'reopen': {
                     'invisible': Eval('state') != 'close',
                     },
+                'lock': {
+                    'invisible': Eval('state') != 'close',
+                    },
                 })
 
     @staticmethod
@@ -97,6 +106,13 @@ class FiscalYear(ModelSQL, ModelView):
     def default_company():
         return Transaction().context.get('company')
 
+    def get_icon(self, name):
+        return {
+            'open': 'tryton-open',
+            'close': 'tryton-close',
+            'locked': 'tryton-readonly',
+            }.get(self.state)
+
     @classmethod
     def validate(cls, years):
         super(FiscalYear, cls).validate(years)
@@ -105,7 +121,10 @@ class FiscalYear(ModelSQL, ModelView):
             year.check_post_move_sequence()
 
     def check_dates(self):
-        cursor = Transaction().connection.cursor()
+        transaction = Transaction()
+        connection = transaction.connection
+        transaction.database.lock(connection, self._table)
+        cursor = connection.cursor()
         table = self.__table__()
         cursor.execute(*table.select(table.id,
                 where=(((table.start_date <= self.start_date)
@@ -223,7 +242,7 @@ class FiscalYear(ModelSQL, ModelView):
                 language = Transaction().language
                 languages = Lang.search([('code', '=', language)])
                 if not languages:
-                    languages = Lang.search([('code', '=', 'en_US')])
+                    languages = Lang.search([('code', '=', 'en')])
                 language, = languages
                 formatted = Lang.strftime(date, language.code, language.date)
                 cls.raise_user_error('no_fiscalyear_date', (formatted,))
@@ -249,10 +268,12 @@ class FiscalYear(ModelSQL, ModelView):
             deferral.fiscalyear = self
             deferral.debit = account.debit
             deferral.credit = account.credit
+            deferral.amount_second_currency = account.amount_second_currency
             return deferral
 
     @classmethod
     @ModelView.button
+    @Workflow.transition('close')
     def close(cls, fiscalyears):
         '''
         Close a fiscal year
@@ -261,6 +282,12 @@ class FiscalYear(ModelSQL, ModelView):
         Period = pool.get('account.period')
         Account = pool.get('account.account')
         Deferral = pool.get('account.account.deferral')
+        transaction = Transaction()
+        database = transaction.database
+        connection = transaction.connection
+
+        # Lock period to be sure no new period will be created in between.
+        database.lock(connection, Period._table)
 
         deferrals = []
         for fiscalyear in fiscalyears:
@@ -271,11 +298,6 @@ class FiscalYear(ModelSQL, ModelView):
                         ]):
                 cls.raise_user_error('close_error', (fiscalyear.rec_name,))
 
-            # First close the fiscalyear to be sure
-            # it will not have new period created between.
-            cls.write([fiscalyear], {
-                'state': 'close',
-                })
             periods = Period.search([
                     ('fiscalyear', '=', fiscalyear.id),
                     ])
@@ -292,6 +314,7 @@ class FiscalYear(ModelSQL, ModelView):
 
     @classmethod
     @ModelView.button
+    @Workflow.transition('open')
     def reopen(cls, fiscalyears):
         '''
         Re-open a fiscal year
@@ -301,7 +324,7 @@ class FiscalYear(ModelSQL, ModelView):
         for fiscalyear in fiscalyears:
             if cls.search([
                         ('start_date', '>=', fiscalyear.end_date),
-                        ('state', '=', 'close'),
+                        ('state', '!=', 'open'),
                         ('company', '=', fiscalyear.company.id),
                         ]):
                 cls.raise_user_error('reopen_error')
@@ -311,20 +334,26 @@ class FiscalYear(ModelSQL, ModelView):
                 ])
             Deferral.delete(deferrals)
 
-            cls.write([fiscalyear], {
-                'state': 'open',
-                })
-
     @classmethod
-    def search_rec_name(cls, name, clause):
-        if clause[1].startswith('!') or clause[1].startswith('not '):
-            bool_op = 'AND'
-        else:
-            bool_op = 'OR'
-        return [bool_op,
-            ('code',) + tuple(clause[1:]),
-            (cls._rec_name,) + tuple(clause[1:]),
-            ]
+    @ModelView.button
+    @Workflow.transition('locked')
+    def lock(cls, fiscalyears):
+        pool = Pool()
+        Period = pool.get('account.period')
+        periods = Period.search([
+                ('fiscalyear', 'in', [f.id for f in fiscalyears]),
+                ])
+        Period.lock(periods)
+
+
+class FiscalYearLine(ModelSQL):
+    'Fiscal Year - Move Line'
+    __name__ = 'account.fiscalyear-account.move.line'
+    _table = 'account_fiscalyear_line_rel'
+    fiscalyear = fields.Many2One('account.fiscalyear', 'Fiscal Year',
+            ondelete='CASCADE', select=True)
+    line = fields.Many2One('account.move.line', 'Line', ondelete='RESTRICT',
+            select=True, required=True)
 
 
 class BalanceNonDeferralStart(ModelView):
@@ -449,27 +478,3 @@ class BalanceNonDeferral(Wizard):
                 ('origin', '=', str(self.start.fiscalyear)),
                 ])
         return action, {}
-
-
-class CloseFiscalYearStart(ModelView):
-    'Close Fiscal Year'
-    __name__ = 'account.fiscalyear.close.start'
-    close_fiscalyear = fields.Many2One('account.fiscalyear',
-            'Fiscal Year to close', required=True,
-            domain=[('state', '!=', 'close')])
-
-
-class CloseFiscalYear(Wizard):
-    'Close Fiscal Year'
-    __name__ = 'account.fiscalyear.close'
-    start = StateView('account.fiscalyear.close.start',
-        'account.fiscalyear_close_start_view_form', [
-            Button('Cancel', 'end', 'tryton-cancel'),
-            Button('Close', 'close', 'tryton-ok', default=True),
-            ])
-    close = StateTransition()
-
-    def transition_close(self):
-        Fiscalyear = Pool().get('account.fiscalyear')
-        Fiscalyear.close([self.start.close_fiscalyear])
-        return 'end'
